@@ -3,7 +3,9 @@ use crate::error::Error;
 use crate::manifest::Edition;
 use serde::de::value::MapAccessDeserializer;
 use serde::de::{self, Visitor};
-use serde::{Deserialize, Deserializer, Serialize, Serializer};
+use serde::de::{Deserialize, Deserializer};
+use serde::ser::{Serialize, Serializer};
+use serde_derive::{Deserialize, Serialize};
 use std::collections::BTreeMap as Map;
 use std::fmt;
 use std::fs;
