@@ -15,7 +15,6 @@ use std::ffi::{OsStr, OsString};
 use std::fs::{self, File};
 use std::mem;
 use std::path::{Path, PathBuf};
-use std::process::Output;
 use std::str;
 
 #[derive(Debug)]
@@ -63,6 +62,14 @@ impl Runner {
 
         if tests.is_empty() {
             message::no_tests_enabled();
+        } else if project.keep_going && !project.has_pass {
+            failures = match self.run_all(&project, tests) {
+                Ok(failures) => failures,
+                Err(err) => {
+                    message::test_fail(err);
+                    len
+                }
+            }
         } else {
             for test in tests {
                 if let Err(err) = test.run(&project) {
@@ -261,6 +268,35 @@ impl Runner {
             },
         }
     }
+
+    fn run_all(&self, project: &Project, tests: Vec<ExpandedTest>) -> Result<usize> {
+        let mut failures = 0;
+        let output = cargo::build_all_tests(project)?;
+        let parsed = parse_cargo_json(&output.stdout);
+        let fallback = Stderr::default();
+
+        for mut t in tests {
+            let show_expected = false;
+            message::begin_test(&t.test, show_expected);
+
+            if t.error.is_none() {
+                t.error = check_exists(&t.test.path).err();
+            }
+
+            if t.error.is_none() {
+                let src_path = project.source_dir.join(&t.test.path);
+                let this_test = parsed.stderrs.get(&src_path).unwrap_or(&fallback);
+                t.error = t.test.check(project, &t.name, this_test, "").err();
+            }
+
+            if let Some(err) = t.error {
+                failures += 1;
+                message::test_fail(err);
+            }
+        }
+
+        Ok(failures)
+    }
 }
 
 impl Test {
@@ -269,13 +305,24 @@ impl Test {
         message::begin_test(self, show_expected);
         check_exists(&self.path)?;
 
-        let mut output = cargo::build_test(project, name)?;
+        let output = cargo::build_test(project, name)?;
+        let parsed = parse_cargo_json(&output.stdout);
         let src_path = project.source_dir.join(&self.path);
-        parse_cargo_json(&src_path, &mut output);
-        let success = output.status.success();
-        let stdout = output.stdout;
+        let fallback = Stderr::default();
+        let this_test = parsed.stderrs.get(&src_path).unwrap_or(&fallback);
+        self.check(project, name, this_test, &parsed.stdout)
+    }
+
+    fn check(
+        &self,
+        project: &Project,
+        name: &Name,
+        result: &Stderr,
+        build_stdout: &str,
+    ) -> Result<()> {
+        let success = result.success;
         let stderr = normalize::diagnostics(
-            output.stderr,
+            &result.stderr,
             Context {
                 krate: &name.0,
                 source_dir: &project.source_dir,
@@ -291,7 +338,7 @@ impl Test {
             Expected::CompileFail => Test::check_compile_fail,
         };
 
-        check(self, project, name, success, stdout, stderr)
+        check(self, project, name, success, build_stdout, stderr)
     }
 
     fn check_pass(
@@ -299,7 +346,7 @@ impl Test {
         project: &Project,
         name: &Name,
         success: bool,
-        build_stdout: Vec<u8>,
+        build_stdout: &str,
         variations: Variations,
     ) -> Result<()> {
         let preferred = variations.preferred();
@@ -309,7 +356,7 @@ impl Test {
         }
 
         let mut output = cargo::run_test(project, name)?;
-        output.stdout.splice(..0, build_stdout);
+        output.stdout.splice(..0, build_stdout.bytes());
         message::output(preferred, &output);
         if output.status.success() {
             Ok(())
@@ -323,14 +370,14 @@ impl Test {
         project: &Project,
         _name: &Name,
         success: bool,
-        build_stdout: Vec<u8>,
+        build_stdout: &str,
         variations: Variations,
     ) -> Result<()> {
         let preferred = variations.preferred();
 
         if success {
             message::should_not_have_compiled();
-            message::fail_output(Fail, &build_stdout);
+            message::fail_output(Fail, build_stdout);
             message::warnings(preferred);
             return Err(Error::ShouldNotHaveCompiled);
         }
@@ -356,7 +403,7 @@ impl Test {
                     fs::write(stderr_path, preferred).map_err(Error::WriteStderr)?;
                 }
             }
-            message::fail_output(Warn, &build_stdout);
+            message::fail_output(Warn, build_stdout);
             return Ok(());
         }
 
@@ -519,15 +566,29 @@ struct RustcMessage {
     level: String,
 }
 
-fn parse_cargo_json(src_path: &Path, output: &mut Output) {
-    let mut diagnostics = String::new();
+struct ParsedOutputs {
+    stdout: String,
+    stderrs: Map<PathBuf, Stderr>,
+}
+
+struct Stderr {
+    success: bool,
+    stderr: String,
+}
+
+impl Default for Stderr {
+    fn default() -> Self {
+        Stderr {
+            success: true,
+            stderr: String::new(),
+        }
+    }
+}
+
+fn parse_cargo_json(stdout: &[u8]) -> ParsedOutputs {
+    let mut map = Map::new();
     let mut nonmessage_stdout = String::new();
-    let mut remaining = match str::from_utf8(&output.stdout) {
-        Ok(remaining) => remaining,
-        // TODO: this should be easy to support if someone needs it, just needs
-        // an alternative to `find` below.
-        Err(_) => return,
-    };
+    let mut remaining = &*String::from_utf8_lossy(stdout);
     while !remaining.is_empty() {
         let begin = match remaining.find("{\"reason\":") {
             Some(begin) => begin,
@@ -541,13 +602,21 @@ fn parse_cargo_json(src_path: &Path, output: &mut Output) {
         };
         let (message, rest) = rest.split_at(len);
         if let Ok(de) = serde_json::from_str::<CargoMessage>(message) {
-            if de.message.level != "failure-note" && de.target.src_path == src_path {
-                diagnostics.push_str(&de.message.rendered);
+            if de.message.level != "failure-note" {
+                let mut entry = map
+                    .entry(de.target.src_path)
+                    .or_insert_with(Stderr::default);
+                if de.message.level == "error" {
+                    entry.success = false;
+                }
+                entry.stderr.push_str(&de.message.rendered);
             }
         }
         remaining = rest;
     }
     nonmessage_stdout.push_str(remaining);
-    output.stdout = Vec::from(nonmessage_stdout);
-    output.stderr = Vec::from(diagnostics);
+    ParsedOutputs {
+        stdout: nonmessage_stdout,
+        stderrs: map,
+    }
 }
